@@ -6,14 +6,16 @@ import logging
 import time
 import warnings
 from itertools import chain
-from typing import TYPE_CHECKING, List, Optional, Tuple, Type
+from typing import TYPE_CHECKING, List, Optional, Tuple, Type, Union
 
 import numpy as np
+from typing_extensions import Literal
 
 import aesara
 import aesara.compile.profiling
-from aesara.compile.io import In, SymbolicInput, SymbolicOutput
-from aesara.compile.ops import deep_copy_op, view_op
+from aesara.compile.io import In, Out, SymbolicInput, SymbolicOutput
+from aesara.compile.ops import deep_copy_op, update_placeholder, view_op
+from aesara.compile.profiling import ProfileStats
 from aesara.configdefaults import config
 from aesara.graph.basic import (
     Constant,
@@ -32,6 +34,7 @@ from aesara.link.utils import raise_with_op
 
 
 if TYPE_CHECKING:
+    from aesara.compile.mode import Mode
     from aesara.link.vm import VM
 
 
@@ -731,10 +734,10 @@ class Function:
                 message = name
             else:
                 message = str(profile.message) + " copy"
-            profile = aesara.compile.profiling.ProfileStats(message=message)
+            profile = ProfileStats(message=message)
             # profile -> object
         elif isinstance(profile, str):
-            profile = aesara.compile.profiling.ProfileStats(message=profile)
+            profile = ProfileStats(message=profile)
 
         f_cpy = maker.__class__(
             inputs=ins,
@@ -824,7 +827,7 @@ class Function:
                     self[i] = value
 
         profile = self.profile
-        t0 = time.time()
+        t0 = time.perf_counter()
 
         output_subset = kwargs.pop("output_subset", None)
         if output_subset is not None and self.output_keys is not None:
@@ -965,7 +968,7 @@ class Function:
                     )
 
         # Do the actual work
-        t0_fn = time.time()
+        t0_fn = time.perf_counter()
         try:
             outputs = (
                 self.vm()
@@ -991,7 +994,7 @@ class Function:
                 # old-style linkers raise their own exceptions
                 raise
 
-        dt_fn = time.time() - t0_fn
+        dt_fn = time.perf_counter() - t0_fn
         self.maker.mode.fn_time += dt_fn
         if profile:
             profile.vm_call_time += dt_fn
@@ -1039,7 +1042,7 @@ class Function:
         #       grep for 'PROFILE_CODE'
         #
 
-        dt_call = time.time() - t0
+        dt_call = time.perf_counter() - t0
         aesara.compile.profiling.total_fct_exec_time += dt_call
         self.maker.mode.call_time += dt_call
         if profile:
@@ -1391,32 +1394,96 @@ class FunctionMaker:
 
     @staticmethod
     def prepare_fgraph(
-        inputs, outputs, additional_outputs, fgraph, rewriter, linker, profile
+        inputs: List[In],
+        outputs: List[Out],
+        additional_outputs: List[Out],
+        fgraph: FunctionGraph,
+        mode: "Mode",
+        profile: Union[Optional[ProfileStats], Literal[False]],
     ):
+        r"""Perform rewrites on a graph, insert `DeepCopyOp`\s, and remove unused updates.
+
+        .. warning::
+
+            The `additional_outputs` list and `fgraph.outputs` are updated in-place by this method.
+
+        Parameters
+        ==========
+        inputs
+            The wrapped inputs.
+        outputs
+            The wrapped outputs (i.e. wrapped with `Out`).
+        additional_outputs
+            Output graphs that essentially serve as updates to mutable `inputs`.
+        fgraph
+            The `FunctionGraph` to be prepared.
+        mode
+            The `Mode` that determines--for example--which rewrites are applied.
+        profile
+            The profile object/setting to use.
+
+        """
+
+        rewriter = mode.optimizer
 
         try:
-            start_rewriter = time.time()
+            start_rewriter = time.perf_counter()
 
             rewriter_profile = None
             rewrite_time = None
 
             with config.change_flags(
+                mode=mode,
                 compute_test_value=config.compute_test_value_opt,
                 traceback__limit=config.traceback__compile_limit,
             ):
                 rewriter_profile = rewriter(fgraph)
 
-                end_rewriter = time.time()
+                end_rewriter = time.perf_counter()
                 rewrite_time = end_rewriter - start_rewriter
                 _logger.debug(f"Rewriting took {rewrite_time:f} seconds")
 
+                fgraph_outputs = tuple(fgraph.outputs)
+                update_mappings = tuple(fgraph.update_mapping.items())
+                outputs_to_remove = []
+                additional_outputs_to_remove = []
+
+                # Remove unused updates
+                for i, (out_idx, in_idx) in enumerate(update_mappings):
+                    update = fgraph_outputs[out_idx]
+
+                    if update.owner and update.owner.op == update_placeholder:
+
+                        # TODO: Consider removing the corresponding
+                        # `FunctionGraph` input when it has no other
+                        # references?
+                        # updated_var = fgraph_inputs[in_idx]
+                        # if not fgraph.clients[updated_var]:
+                        #     fgraph.remove_input(updated_var)
+
+                        # Remove the update entry from the wrapped inputs
+                        inputs[in_idx].update = None
+
+                        # We assume that the orders of `fgraph.update_mapping` and
+                        # `additional_outputs` correspond (and they should)
+                        additional_outputs_to_remove.append(additional_outputs[i])
+
+                        outputs_to_remove.append(fgraph.outputs[out_idx])
+
+                for add_out, out in zip(
+                    additional_outputs_to_remove, outputs_to_remove
+                ):
+                    additional_outputs.remove(add_out)
+                    fgraph_out_idx = fgraph.outputs.index(out)
+                    fgraph.remove_output(fgraph_out_idx)
+
                 # Add deep copy to respect the memory interface
                 insert_deepcopy(fgraph, inputs, outputs + additional_outputs)
         finally:
 
             # If the rewriter got interrupted
             if rewrite_time is None:
-                end_rewriter = time.time()
+                end_rewriter = time.perf_counter()
                 rewrite_time = end_rewriter - start_rewriter
 
             aesara.compile.profiling.total_graph_rewrite_time += rewrite_time
@@ -1440,7 +1507,7 @@ class FunctionMaker:
                     stacklevel=3,
                 )
 
-        if not hasattr(linker, "accept"):
+        if not hasattr(mode.linker, "accept"):
             raise ValueError(
                 "'linker' parameter of FunctionMaker should be "
                 f"a Linker with an accept method or one of {list(aesara.compile.mode.predefined_linkers.keys())}"
@@ -1511,12 +1578,8 @@ class FunctionMaker:
 
         self.fgraph = fgraph
 
-        rewriter, linker = mode.optimizer, copy.copy(mode.linker)
-
         if not no_fgraph_prep:
-            self.prepare_fgraph(
-                inputs, outputs, found_updates, fgraph, rewriter, linker, profile
-            )
+            self.prepare_fgraph(inputs, outputs, found_updates, fgraph, mode, profile)
 
         assert len(fgraph.outputs) == len(outputs + found_updates)
 
@@ -1528,6 +1591,8 @@ class FunctionMaker:
             if not spec.borrow
         ]
 
+        linker = copy.copy(mode.linker)
+
         if no_borrow:
             self.linker = linker.accept(
                 fgraph,
@@ -1645,7 +1710,7 @@ class FunctionMaker:
             defaults.append((required, refeed, storage))
 
         # Get a function instance
-        start_linker = time.time()
+        start_linker = time.perf_counter()
         start_import_time = aesara.link.c.cmodule.import_time
 
         with config.change_flags(traceback__limit=config.traceback__compile_limit):
@@ -1653,7 +1718,7 @@ class FunctionMaker:
                 input_storage=input_storage_lists, storage_map=storage_map
             )
 
-        end_linker = time.time()
+        end_linker = time.perf_counter()
 
         linker_time = end_linker - start_linker
         aesara.compile.profiling.total_time_linker += linker_time
@@ -1725,7 +1790,7 @@ def orig_function(
 
     """
 
-    t1 = time.time()
+    t1 = time.perf_counter()
     mode = aesara.compile.mode.get_mode(mode)
 
     inputs = list(map(convert_function_input, inputs))
@@ -1758,7 +1823,7 @@ def orig_function(
         with config.change_flags(compute_test_value="off"):
             fn = m.create(defaults)
     finally:
-        t2 = time.time()
+        t2 = time.perf_counter()
         if fn and profile:
             profile.compile_time += t2 - t1
             # TODO: append
