@@ -492,7 +492,7 @@ def grad(
         respect to the output, then a zero variable is returned.
 
     """
-    t0 = time.time()
+    t0 = time.perf_counter()
 
     if cost is None:
         if known_grads is None:
@@ -643,7 +643,7 @@ def grad(
             else:
                 assert return_disconnected.lower() == "disconnected"
 
-    t1 = time.time()
+    t1 = time.perf_counter()
     global grad_time
     grad_time += t1 - t0
 
@@ -1802,13 +1802,11 @@ def verify_grad(
         mode=mode,
     )
 
-    tensor_pt = [
-        aesara.tensor.type.TensorType(
-            aesara.tensor.as_tensor_variable(p).dtype,
-            aesara.tensor.as_tensor_variable(p).broadcastable,
-        )(name=f"input {i}")
-        for i, p in enumerate(pt)
-    ]
+    tensor_pt = []
+    for i, p in enumerate(pt):
+        p_t = aesara.tensor.as_tensor_variable(p).type()
+        p_t.name = f"input {i}"
+        tensor_pt.append(p_t)
 
     # fun can be either a function or an actual Op instance
     o_output = fun(*tensor_pt)
