@@ -72,7 +72,7 @@ class FunctionGraph(MetaObject):
         outputs: Optional[Sequence[Variable]] = None,
         features: Optional[Sequence[Feature]] = None,
         clone: bool = True,
-        update_mapping: Optional[Dict[Variable, Variable]] = None,
+        update_mapping: Optional[Dict[int, int]] = None,
         **clone_kwds,
     ):
         """
@@ -90,8 +90,8 @@ class FunctionGraph(MetaObject):
         clone
             If ``True``, the graph will be cloned.
         update_mapping
-            Mapping between the `inputs` with updates and the `outputs`
-            corresponding to their updates.
+            Mapping between the `outputs` indices of update graphs and the `inputs`
+            indices that are updated with the former's values.
         clone_kwds
             Keywords passed to `clone_get_equiv` when `clone` is ``True``.
         """
@@ -153,7 +153,18 @@ class FunctionGraph(MetaObject):
             self.add_output(output, reason="init")
 
         self.profile = None
-        self.update_mapping = update_mapping
+
+        self.set_update_mapping(update_mapping)
+
+    def set_update_mapping(self, update_mapping):
+        self.update_mapping = {}
+        self.inv_update_mapping = {}
+
+        if update_mapping:
+            for k, v in update_mapping.items():
+                self.update_mapping[k] = v
+                # `update_mapping` should be bijective, making this reasonable
+                self.inv_update_mapping[v] = k
 
     def add_output(
         self, var: Variable, reason: Optional[str] = None, import_missing: bool = False
@@ -541,12 +552,22 @@ class FunctionGraph(MetaObject):
 
         new_idx = 0
         for (out, old_idx) in old_idx_mappings:
+            # We need to update `update_mappings`, as well
+            map_in_idx = self.update_mapping.pop(old_idx, None)
+            self.inv_update_mapping.pop(map_in_idx, None)
+
             if old_idx == idx:
                 continue
+
+            if map_in_idx is not None:
+                self.update_mapping[new_idx] = map_in_idx
+                self.inv_update_mapping[map_in_idx] = new_idx
+
             out_clients = self.clients[out]
             arrow: ClientType = ("output", old_idx)
             arrow_idx = out_clients.index(arrow)
             out_clients[arrow_idx] = ("output", new_idx)
+
             new_idx += 1
 
     def remove_node(self, node: Apply, reason: Optional[str] = None):
@@ -637,8 +658,19 @@ class FunctionGraph(MetaObject):
 
     def remove_input(self, input_idx: int, reason: Optional[str] = None):
         """Remove the input at index `input_idx`."""
+
         var = self.inputs.pop(input_idx)
 
+        for in_idx, out_idx in tuple(self.inv_update_mapping.items()):
+            if in_idx == input_idx:
+                del self.update_mapping[out_idx]
+                del self.inv_update_mapping[in_idx]
+            elif in_idx > input_idx:
+                new_in_idx = in_idx - 1
+                self.update_mapping[out_idx] = new_in_idx
+                del self.inv_update_mapping[in_idx]
+                self.inv_update_mapping[new_in_idx] = out_idx
+
         for client, idx in list(self.clients[var]):
             if client == "output":
                 out_var = self.outputs[idx]
@@ -712,7 +744,7 @@ class FunctionGraph(MetaObject):
         a method called after name.
 
         """
-        t0 = time.time()
+        t0 = time.perf_counter()
         for feature in self._features:
             try:
                 fn = getattr(feature, name)
@@ -721,10 +753,10 @@ class FunctionGraph(MetaObject):
                 # try; the AttributeError really must come from feature.${name}
                 # not existing
                 continue
-            tf0 = time.time()
+            tf0 = time.perf_counter()
             fn(self, *args, **kwargs)
-            self.execute_callbacks_times[feature] += time.time() - tf0
-        self.execute_callbacks_time += time.time() - t0
+            self.execute_callbacks_times[feature] += time.perf_counter() - tf0
+        self.execute_callbacks_time += time.perf_counter() - t0
 
     def collect_callbacks(self, name: str, *args) -> Dict[Feature, Any]:
         """Collects callbacks
