@@ -10,7 +10,11 @@ from aesara import config
 from aesara.compile.ops import ViewOp
 from aesara.graph.basic import Variable
 from aesara.link.numba.dispatch import basic as numba_basic
-from aesara.link.numba.dispatch.basic import create_numba_signature, numba_funcify
+from aesara.link.numba.dispatch.basic import (
+    _numba_funcify,
+    create_numba_signature,
+    numba_funcify,
+)
 from aesara.link.utils import (
     compile_function_src,
     get_name_for_object,
@@ -31,7 +35,7 @@ from aesara.scalar.basic import (
 from aesara.scalar.math import Erf, Erfc, GammaLn, Log1mexp, Sigmoid
 
 
-@numba_funcify.register(ScalarOp)
+@_numba_funcify.register(ScalarOp)
 def numba_funcify_ScalarOp(op, node, **kwargs):
     # TODO: Do we need to cache these functions so that we don't end up
     # compiling the same Numba function over and over again?
@@ -126,7 +130,7 @@ def {scalar_op_fn_name}({', '.join(input_names)}):
     )(scalar_op_fn)
 
 
-@numba_funcify.register(Switch)
+@_numba_funcify.register(Switch)
 def numba_funcify_Switch(op, node, **kwargs):
     @numba_basic.numba_njit(inline="always")
     def switch(condition, x, y):
@@ -154,7 +158,7 @@ def {binary_op_name}({input_signature}):
     return nary_fn
 
 
-@numba_funcify.register(Add)
+@_numba_funcify.register(Add)
 def numba_funcify_Add(op, node, **kwargs):
 
     signature = create_numba_signature(node, force_scalar=True)
@@ -166,7 +170,7 @@ def numba_funcify_Add(op, node, **kwargs):
     )(nary_add_fn)
 
 
-@numba_funcify.register(Mul)
+@_numba_funcify.register(Mul)
 def numba_funcify_Mul(op, node, **kwargs):
 
     signature = create_numba_signature(node, force_scalar=True)
@@ -178,7 +182,7 @@ def numba_funcify_Mul(op, node, **kwargs):
     )(nary_mul_fn)
 
 
-@numba_funcify.register(Cast)
+@_numba_funcify.register(Cast)
 def numba_funcify_Cast(op, node, **kwargs):
 
     dtype = np.dtype(op.o_type.dtype)
@@ -190,8 +194,8 @@ def numba_funcify_Cast(op, node, **kwargs):
     return cast
 
 
-@numba_funcify.register(Identity)
-@numba_funcify.register(ViewOp)
+@_numba_funcify.register(Identity)
+@_numba_funcify.register(ViewOp)
 def numba_funcify_ViewOp(op, **kwargs):
     @numba_basic.numba_njit(inline="always")
     def viewop(x):
@@ -200,7 +204,7 @@ def numba_funcify_ViewOp(op, **kwargs):
     return viewop
 
 
-@numba_funcify.register(Clip)
+@_numba_funcify.register(Clip)
 def numba_funcify_Clip(op, **kwargs):
     @numba_basic.numba_njit
     def clip(_x, _min, _max):
@@ -218,16 +222,19 @@ def numba_funcify_Clip(op, **kwargs):
     return clip
 
 
-@numba_funcify.register(Composite)
+@_numba_funcify.register(Composite)
 def numba_funcify_Composite(op, node, **kwargs):
     signature = create_numba_signature(node, force_scalar=True)
+
+    _ = kwargs.pop("storage_map", None)
+
     composite_fn = numba_basic.numba_njit(signature, fastmath=config.numba__fastmath)(
         numba_funcify(op.fgraph, squeeze_output=True, **kwargs)
     )
     return composite_fn
 
 
-@numba_funcify.register(Second)
+@_numba_funcify.register(Second)
 def numba_funcify_Second(op, node, **kwargs):
     @numba_basic.numba_njit(inline="always")
     def second(x, y):
@@ -236,7 +243,7 @@ def numba_funcify_Second(op, node, **kwargs):
     return second
 
 
-@numba_funcify.register(Reciprocal)
+@_numba_funcify.register(Reciprocal)
 def numba_funcify_Reciprocal(op, node, **kwargs):
     @numba_basic.numba_njit(inline="always")
     def reciprocal(x):
@@ -247,7 +254,7 @@ def numba_funcify_Reciprocal(op, node, **kwargs):
     return reciprocal
 
 
-@numba_funcify.register(Sigmoid)
+@_numba_funcify.register(Sigmoid)
 def numba_funcify_Sigmoid(op, node, **kwargs):
     @numba_basic.numba_njit(inline="always", fastmath=config.numba__fastmath)
     def sigmoid(x):
@@ -256,7 +263,7 @@ def numba_funcify_Sigmoid(op, node, **kwargs):
     return sigmoid
 
 
-@numba_funcify.register(GammaLn)
+@_numba_funcify.register(GammaLn)
 def numba_funcify_GammaLn(op, node, **kwargs):
     @numba_basic.numba_njit(inline="always", fastmath=config.numba__fastmath)
     def gammaln(x):
@@ -265,7 +272,7 @@ def numba_funcify_GammaLn(op, node, **kwargs):
     return gammaln
 
 
-@numba_funcify.register(Log1mexp)
+@_numba_funcify.register(Log1mexp)
 def numba_funcify_Log1mexp(op, node, **kwargs):
     @numba_basic.numba_njit(inline="always", fastmath=config.numba__fastmath)
     def logp1mexp(x):
@@ -277,7 +284,7 @@ def numba_funcify_Log1mexp(op, node, **kwargs):
     return logp1mexp
 
 
-@numba_funcify.register(Erf)
+@_numba_funcify.register(Erf)
 def numba_funcify_Erf(op, **kwargs):
     @numba_basic.numba_njit(inline="always", fastmath=config.numba__fastmath)
     def erf(x):
@@ -286,7 +293,7 @@ def numba_funcify_Erf(op, **kwargs):
     return erf
 
 
-@numba_funcify.register(Erfc)
+@_numba_funcify.register(Erfc)
 def numba_funcify_Erfc(op, **kwargs):
     @numba_basic.numba_njit(inline="always", fastmath=config.numba__fastmath)
     def erfc(x):
