@@ -1,5 +1,5 @@
 from copy import copy
-from typing import List, Tuple, Union
+from typing import List, Tuple
 
 import numpy as np
 
@@ -62,17 +62,17 @@ class DimShuffle(ExternalCOp):
     If `j = new_order[i]` is an index, the output's ith dimension
     will be the input's jth dimension.
     If `new_order[i]` is `x`, the output's ith dimension will
-    be 1 and Broadcast operations will be allowed to do broadcasting
+    be 1 and broadcast operations will be allowed to do broadcasting
     over that dimension.
 
-    If `input.broadcastable[i] == False` then `i` must be found in new_order.
+    If `input.type.shape[i] != 1` then `i` must be found in `new_order`.
     Broadcastable dimensions, on the other hand, can be discarded.
 
     .. code-block:: python
 
         DimShuffle((False, False, False), ['x', 2, 'x', 0, 1])
 
-    This op will only work on 3d tensors with no broadcastable
+    This `Op` will only work on 3d tensors with no broadcastable
     dimensions.  The first dimension will be broadcastable,
     then we will have the third dimension of the input tensor as
     the second of the resulting tensor, etc. If the tensor has
@@ -83,7 +83,7 @@ class DimShuffle(ExternalCOp):
 
         DimShuffle((True, False), [1])
 
-    This op will only work on 2d tensors with the first dimension
+    This `Op` will only work on 2d tensors with the first dimension
     broadcastable.
     The second dimension of the input tensor will be the first dimension of
     the resulting tensor.
@@ -186,7 +186,7 @@ class DimShuffle(ExternalCOp):
 
     def make_node(self, _input):
         input = as_tensor_variable(_input)
-        ib = tuple(input.type.broadcastable)
+        ib = tuple(s == 1 for s in input.type.shape)
         if ib != self.input_broadcastable:
             if len(ib) != len(self.input_broadcastable):
                 raise TypeError(
@@ -258,7 +258,7 @@ class DimShuffle(ExternalCOp):
         (x,) = inp
         (gz,) = grads
         gz = as_tensor_variable(gz)
-        grad_order = ["x"] * len(x.type.broadcastable)
+        grad_order = ["x"] * x.type.ndim
         for i, v in enumerate(self.new_order):
             if v != "x":
                 grad_order[v] = i
@@ -269,7 +269,7 @@ class DimShuffle(ExternalCOp):
             return [inp[0].zeros_like(dtype=config.floatX)]
         else:
             return [
-                DimShuffle(gz.type.broadcastable, grad_order)(
+                DimShuffle(tuple(s == 1 for s in gz.type.shape), grad_order)(
                     Elemwise(scalar_identity)(gz)
                 )
             ]
@@ -325,7 +325,7 @@ class Elemwise(OpenMPOp):
     -``Elemwise(add, {0 : 1})``: represents ``+=`` on the second argument ``y += x``
     -``Elemwise(mul)(np.random.random((10, 5)), np.random.random((1, 5)))``:
     the second input is completed along the first dimension to match the first input
-    -``Elemwise(true_div)(np.random.random(10, 5), np.random.random(10, 1))``: same but along the
+    -``Elemwise(true_divide)(np.random.random(10, 5), np.random.random(10, 1))``: same but along the
     second dimension
     -``Elemwise(int_div)(np.random.random((1, 5)), np.random.random((10, 1)))``:
     the output has size ``(10, 5)``.
@@ -406,7 +406,7 @@ class Elemwise(OpenMPOp):
                 # TODO: use LComplete instead
                 args.append(
                     dim_shuffle(
-                        input.type.broadcastable,
+                        tuple(1 if s == 1 else None for s in input.type.shape),
                         ["x"] * difference + list(range(length)),
                     )(input)
                 )
@@ -452,11 +452,11 @@ class Elemwise(OpenMPOp):
         inplace_pattern = self.inplace_pattern
         if inplace_pattern:
             for overwriter, overwritten in inplace_pattern.items():
-                for ob, ib in zip(
+                for out_s, in_s in zip(
                     out_shapes[overwriter],
-                    inputs[overwritten].type.broadcastable,
+                    inputs[overwritten].type.shape,
                 ):
-                    if ib and not ob == 1:
+                    if in_s == 1 and out_s != 1:
                         raise ValueError(
                             "Operation cannot be done inplace on an input "
                             "with broadcasted dimensions."
@@ -578,8 +578,8 @@ class Elemwise(OpenMPOp):
             # TODO: only count dimensions that were effectively broadcasted
             to_sum = [
                 j
-                for j, bcast in enumerate(ipt.type.broadcastable)
-                if bcast and not outs[0].broadcastable[j]
+                for j, in_s in enumerate(ipt.type.shape)
+                if in_s == 1 and outs[0].type.shape[j] != 1
             ]
 
             if to_sum:
@@ -614,7 +614,7 @@ class Elemwise(OpenMPOp):
                 f"{str(self.scalar_op)}.grad returned {str(type(scalar_igrads))} instead of list or tuple"
             )
 
-        nd = len(inputs[0].type.broadcastable)  # this is the same for everyone
+        nd = inputs[0].type.ndim  # this is the same for everyone
 
         def transform(r):
             # From a graph of ScalarOps, make a graph of Broadcast ops.
@@ -897,7 +897,7 @@ class Elemwise(OpenMPOp):
         # for each input:
         # same as range(ndim), but with 'x' at all broadcastable positions
         orders = [
-            [x and "x" or i for i, x in enumerate(input.type.broadcastable)]
+            [s == 1 and "x" or i for i, s in enumerate(input.type.shape)]
             for input in inputs
         ]
 
@@ -920,7 +920,7 @@ class Elemwise(OpenMPOp):
             [
                 f"PyArray_ISFORTRAN({arr})"
                 for arr, var in z
-                if not all(var.broadcastable)
+                if not all(s == 1 for s in var.type.shape)
             ]
         )
         # If it is a scalar, make it c contig to prevent problem with
@@ -1005,7 +1005,7 @@ class Elemwise(OpenMPOp):
             or
             # Use simpler code when output ndim == 0 or 1
             # or for broadcated scalar.
-            all(node.outputs[0].broadcastable)
+            all(s == 1 for s in node.outputs[0].type.shape)
         ):
             if nnested:
                 all_code = [("", "")] * (nnested - 1) + [("", code)] + [""]
@@ -1077,7 +1077,7 @@ class Elemwise(OpenMPOp):
             all(o.ndim >= 1 for o in node.outputs)
             and
             # Don't use the contig code for broadcasted scalar.
-            not all(node.outputs[0].broadcastable)
+            not all(s == 1 for s in node.outputs[0].type.shape)
         ):
             contig = None
             try:
@@ -1110,7 +1110,7 @@ class Elemwise(OpenMPOp):
                     """
                     index = ""
                     for x, var in zip(inames + onames, inputs + node.outputs):
-                        if not all(var.broadcastable):
+                        if not all(s == 1 for s in var.type.shape):
                             contig += (
                                 """
             dtype_%(x)s * %(x)s_ptr = (dtype_%(x)s*) PyArray_DATA(%(x)s);
@@ -1144,18 +1144,19 @@ class Elemwise(OpenMPOp):
                     )
             if contig is not None:
                 z = list(zip(inames + onames, inputs + node.outputs))
+                all_broadcastable = all(s == 1 for s in var.type.shape)
                 cond1 = " && ".join(
                     [
                         "PyArray_ISCONTIGUOUS(%s)" % arr
                         for arr, var in z
-                        if not all(var.broadcastable)
+                        if not all_broadcastable
                     ]
                 )
                 cond2 = " && ".join(
                     [
                         "PyArray_ISFORTRAN(%s)" % arr
                         for arr, var in z
-                        if not all(var.broadcastable)
+                        if not all_broadcastable
                     ]
                 )
                 loop = (
@@ -1256,33 +1257,61 @@ class CAReduce(COp):
 
     """
 
-    __props__: Union[
-        Tuple[str], Tuple[str, str], Tuple[str, str, str], Tuple[str, str, str, str]
-    ] = ("scalar_op", "axis")
+    __props__ = ("scalar_op", "axis", "dtype", "acc_dtype", "upcast_discrete_output")
 
-    def __init__(self, scalar_op, axis=None):
+    def __init__(
+        self,
+        scalar_op,
+        axis=None,
+        dtype=None,
+        acc_dtype=None,
+        upcast_discrete_output=False,
+    ):
         """
 
         Parameters
         ----------
         scalar_op
-            A binary scalar `Op` with only one output. It must be commutative
-            and associative.
+            A binary scalar `Op` with only one output.
+            It must be commutative and associative.
         axis
-            - The dimension along which we want to reduce
-            - List of dimensions that we want to reduce
-            - If ``None``, all dimensions are reduced
+            - the dimension along which we want to reduce
+            - list of dimensions that we want to reduce
+            - if ``None``, all dimensions are reduced
+        dtype
+            The dtype of the returned tensor. If ``None``, then we use the default
+            dtype which is the same as the input array's dtype except when
+            `upcast_discrete_output` is ``True`` and the following holds:
+
+            - the input dtype is a signed integer of precision < 64 bit, in which
+            case we use int64
+            - the input dtype is an unsigned integer of precision < 64 bit, in
+            which case we use uint64
+
+            This default dtype does _not_ depend on the value of `acc_dtype`.
+            This behavior is similar in spirit to that of NumPy, except that
+            NumPy uses the default machine integer while we always use 64 bit
+            integers to avoid platform-dependent behavior.
+        acc_dtype
+            The dtype of the internal accumulator.
+            If ``None`` (default), we use the dtype in the list below,
+            or the input dtype if its precision is higher:
+
+            - for int dtypes, we use at least int64;
+            - for uint dtypes, we use at least uint64;
+            - for float dtypes, we use at least float64;
+            - for complex dtypes, we use at least complex128.
+        upcast_discrete_output
+            See
 
         """
         if scalar_op.nin not in (-1, 2) or scalar_op.nout != 1:
             raise NotImplementedError(
-                "CAReduce only supports binary functions with a single " "output."
+                "CAReduce only supports binary functions with a single output."
             )
 
         self.axis = None
-        self.ufunc_is_vectorized = False
         self.scalar_op = scalar_op
-        self.set_ufunc(scalar_op)
 
         if axis is not None:
             if isinstance(axis, (int, np.integer)) or (
@@ -1292,63 +1321,179 @@ class CAReduce(COp):
             else:
                 self.axis = tuple(axis)
 
-    def set_ufunc(self, scalar_op):
-        if hasattr(scalar_op, "nfunc_spec") and hasattr(np, scalar_op.nfunc_spec[0]):
-            self.ufunc = getattr(np, scalar_op.nfunc_spec[0])
-        else:
-            self.ufunc = np.frompyfunc(scalar_op.impl, 2, 1)
-            self.ufunc_is_vectorized = True
+        self.dtype = dtype
+        self.acc_dtype = acc_dtype
+        self.upcast_discrete_output = upcast_discrete_output
 
-    def _output_dtype(self, input_dtype):
-        return input_dtype
+    @property
+    def ufunc(self):
+        if hasattr(self, "_ufunc"):
+            return self._ufunc
+
+        if hasattr(self.scalar_op, "nfunc_spec") and hasattr(
+            np, self.scalar_op.nfunc_spec[0]
+        ):
+            self._ufunc = getattr(np, self.scalar_op.nfunc_spec[0])
+        else:
+            self._ufunc = np.frompyfunc(
+                self.scalar_op.impl, 2, 1, identity=self.scalar_op.identity
+            )
+
+        return self._ufunc
+
+    def _output_dtype(self, idtype):
+
+        if not self.upcast_discrete_output:
+            return idtype
+
+        dtype = self.dtype
+
+        if dtype == "OLD":
+            return dict(
+                int8="int32",
+                int16="int32",
+                int32="int64",
+                uint8="uint32",
+                uint16="uint32",
+                uint32="uint64",
+            ).get(idtype, idtype)
+        elif dtype is None:
+            # If input has a discrete dtype, upcast it to 64
+            return dict(
+                bool="int64",
+                int8="int64",
+                int16="int64",
+                int32="int64",
+                uint8="uint64",
+                uint16="uint64",
+                uint32="uint64",
+            ).get(idtype, idtype)
+        else:
+            # The important is that the accumulator dtype does not
+            # lose precision. Then, the result can be downcasted.
+            return dtype
+
+    def _acc_dtype(self, idtype):
+        acc_dtype = self.acc_dtype
+        if acc_dtype is None:
+            return dict(
+                bool="int64",
+                int8="int64",
+                int16="int64",
+                int32="int64",
+                uint8="uint64",
+                uint16="uint64",
+                uint32="uint64",
+                float16="float32",
+                float32="float64",
+                complex64="complex128",
+            ).get(idtype, idtype)
+        elif acc_dtype in continuous_dtypes and idtype in discrete_dtypes:
+            # Specifying a continuous accumulator for discrete input is OK
+            return acc_dtype
+        else:
+            # The conversion has to be considered an upcast.
+            upcasted_dtype = upcast(idtype, acc_dtype)
+            if acc_dtype != upcasted_dtype:
+                raise TypeError(
+                    f"Cannot build {self} node with input dtype {idtype} "
+                    f"and acc_dtype {acc_dtype}, as precision would be lost. "
+                    "To correct this error, you can:\n"
+                    "  - not specify acc_dtype, or\n"
+                    f"  - use an acc_dtype at least as precise as {upcasted_dtype}.\n"
+                    '  - specify "dtype" instead of "acc_dtype", so '
+                    "the reduction will be precise, but the result will "
+                    'be casted into "dtype" at the end.\n'
+                    "If you are expecting the precision loss, you can "
+                    f'use tensor.cast(..., dtype="{acc_dtype}"), on your input.'
+                )
+            return acc_dtype
 
     def make_node(self, input):
         input = as_tensor_variable(input)
         inp_dims = input.type.ndim
-        inp_bdcast = input.type.broadcastable
         inp_dtype = input.type.dtype
 
+        # We need to redefine make_node so that, if self.dtype is None,
+        # we can infer what dtype should be, and create a node from an Op
+        # of the appropriate dtype.
+        dtype = self._output_dtype(inp_dtype)
+        acc_dtype = self._acc_dtype(inp_dtype)
+
+        assert dtype is not None
+        assert acc_dtype is not None
+
         axis = self.axis
-        if axis is None:
-            axis = list(range(inp_dims))
 
-        copy_op = any(a < 0 for a in axis)
         # scalar inputs are treated as 1D regarding axis in this `Op`
-        try:
-            axis = np.core.numeric.normalize_axis_tuple(axis, ndim=max(1, inp_dims))
-        except np.AxisError:
-            raise np.AxisError(axis, ndim=inp_dims)
+        if axis is not None:
+            try:
+                axis = np.core.numeric.normalize_axis_tuple(axis, ndim=max(1, inp_dims))
+            except np.AxisError:
+                raise np.AxisError(axis, ndim=inp_dims)
 
-        # We can't call self.__class__() as there is a class that
-        # inherits from CAReduce that doesn't have the same signature
-        if copy_op:
-            op = copy(self)
-            op.set_ufunc(op.scalar_op)
-            assert len(axis) == len(self.axis)
-            op.axis = tuple(axis)
+            out_shape = tuple(
+                s for i, s in enumerate(input.type.shape) if i not in axis
+            )
+        else:
+            out_shape = ()
+
+        if (
+            (axis is not None and any(a < 0 for a in axis))
+            or dtype != self.dtype
+            or acc_dtype != self.acc_dtype
+        ):
+            op = self.clone(axis=axis, dtype=dtype, acc_dtype=acc_dtype)
         else:
             op = self
 
-        broadcastable = [x for i, x in enumerate(inp_bdcast) if i not in axis]
-
-        output = TensorType(dtype=self._output_dtype(inp_dtype), shape=broadcastable)()
+        output = TensorType(dtype=dtype, shape=out_shape)()
 
         return Apply(op, [input], [output])
 
-    def __getstate__(self):
-        d = copy(self.__dict__)
-        d.pop("ufunc", None)
-        return d
+    def clone(
+        self,
+        axis=None,
+        dtype=None,
+        acc_dtype=None,
+        upcast_discrete_output=None,
+        **kwargs,
+    ):
+        if axis is None:
+            axis = self.axis
+        if dtype is None:
+            dtype = self.dtype
+        if acc_dtype is None:
+            acc_dtype = self.acc_dtype
+        if upcast_discrete_output is None:
+            upcast_discrete_output = self.upcast_discrete_output
 
-    def __setstate__(self, d):
-        self.__dict__.update(d)
-        self.set_ufunc(self.scalar_op)
+        res = type(self)(
+            self.scalar_op,
+            axis=axis,
+            dtype=dtype,
+            acc_dtype=acc_dtype,
+            upcast_discrete_output=None,
+            **kwargs,
+        )
+
+        return res
 
     def __str__(self):
         prefix = f"{type(self).__name__}{{{self.scalar_op}}}"
+        extra_params = []
+
         if self.axis is not None:
-            axes_str = ", ".join(str(x) for x in self.axis)
-            return f"{prefix}{{{axes_str}}}"
+            axis = ", ".join(str(x) for x in self.axis)
+            extra_params.append(f"axis=[{axis}]")
+
+        if self.acc_dtype:
+            extra_params.append(f"acc_dtype={self.acc_dtype}")
+
+        extra_params_str = ", ".join(extra_params)
+
+        if extra_params_str:
+            return f"{prefix}{{{extra_params_str}}}"
         else:
             return f"{prefix}"
 
@@ -1356,44 +1501,28 @@ class CAReduce(COp):
         (input,) = inp
         (output,) = out
         axis = self.axis
-        if axis is None:
-            axis = list(range(input.ndim))
 
-        if hasattr(self, "acc_dtype") and self.acc_dtype is not None:
+        out_dtype = node.outputs[0].type.dtype
+
+        if self.acc_dtype is not None:
             acc_dtype = self.acc_dtype
         else:
-            acc_dtype = node.outputs[0].type.dtype
+            acc_dtype = out_dtype
 
-        variable = np.array(input, dtype=acc_dtype)
+        # out_dtype = self.dtype if self.dtype and self.dtype != "OLD" else out_dtype
 
-        if axis:
-            # Reducing functions built using np.frompyfunc() do not
-            # support reduction along multiple axes. Hence loop through
-            # each, otherwise numpy's inbuilt reduction functions
-            # support reduction along multiple axes directly.
-            if self.ufunc_is_vectorized:
-                to_reduce = reversed(sorted(axis))
-                for dimension in to_reduce:
-                    variable = self.ufunc.reduce(variable, dimension, dtype=acc_dtype)
-            else:
-                variable = self.ufunc.reduce(variable, axis=tuple(axis))
-            output[0] = _asarray(variable, dtype=node.outputs[0].type.dtype)
-        else:
-            # Force a copy
-            output[0] = np.array(variable, copy=True, dtype=node.outputs[0].type.dtype)
+        input = np.array(input, dtype=acc_dtype)
+
+        out = self.ufunc.reduce(input, axis=axis, dtype=acc_dtype)
+
+        output[0] = _asarray(out, dtype=out_dtype)
 
     def infer_shape(self, fgraph, node, shapes):
         (ishape,) = shapes
         axis = self.axis
         if axis is None:
             return ((),)
-        return (
-            [
-                ishape[i]
-                for (i, b) in enumerate(node.inputs[0].type.broadcastable)
-                if i not in axis
-            ],
-        )
+        return ([ishape[i] for i in range(node.inputs[0].type.ndim) if i not in axis],)
 
     def _c_all(self, node, name, inames, onames, sub):
 
@@ -1411,14 +1540,14 @@ class CAReduce(COp):
         if acc_dtype is not None:
             if acc_dtype == "float16":
                 raise MethodNotDefined("no c_code for float16")
-            acc_type = TensorType(shape=node.outputs[0].broadcastable, dtype=acc_dtype)
+            acc_type = TensorType(shape=node.outputs[0].type.shape, dtype=acc_dtype)
             adtype = acc_type.dtype_specs()[1]
         else:
             adtype = odtype
 
         axis = self.axis
         if axis is None:
-            axis = list(range(len(input.type.broadcastable)))
+            axis = list(range(input.type.ndim))
 
         if len(axis) == 0:
             # The acc_dtype is never a downcast compared to the input dtype
@@ -1592,176 +1721,6 @@ class CAReduce(COp):
             return ()
 
 
-class CAReduceDtype(CAReduce):
-    """A subclass of `CAReduce` that accepts an additional output "dtype" parameter.
-
-    It also accepts an optional `acc_dtype`, which specifies the dtype that
-    will be used for the accumulation.  The accumulation will be done using an
-    array of dtype `acc_dtype`, then it will be cast into `dtype` and returned.
-
-    If no `dtype` is provided, one will be inferred so as not to lose
-    too much precision.
-
-    """
-
-    __props__: Union[Tuple[str, str, str], Tuple[str, str, str, str]] = (
-        "scalar_op",
-        "axis",
-        "dtype",
-        "acc_dtype",
-    )
-
-    def __init__(self, scalar_op, axis=None, dtype=None, acc_dtype=None):
-        """
-
-        Parameters
-        ----------
-        scalar_op
-            A binary scalar `Op` with only one output.
-            It must be commutative and associative.
-        axis
-            * the dimension along which we want to reduce
-            * list of dimensions that we want to reduce
-            * if ``None``, all dimensions are reduced
-        dtype
-            The dtype of the returned tensor. If ``None``, then we use the default
-            dtype which is the same as the input array's dtype except when:
-
-            * the input dtype is a signed integer of precision < 64 bit, in which
-            case we use int64
-            * the input dtype is an unsigned integer of precision < 64 bit, in
-            which case we use uint64
-
-            This default dtype does _not_ depend on the value of `acc_dtype`.
-            This behavior is similar in spirit to that of NumPy, except that
-            NumPy uses the default machine integer while we always use 64 bit
-            integers to avoid platform-dependent behavior.
-        acc_dtype
-            The dtype of the internal accumulator.
-            If ``None`` (default), we use the dtype in the list below,
-            or the input dtype if its precision is higher:
-
-            * for int dtypes, we use at least int64;
-            * for uint dtypes, we use at least uint64;
-            * for float dtypes, we use at least float64;
-            * for complex dtypes, we use at least complex128.
-
-        """
-        super().__init__(scalar_op, axis=axis)
-        self.dtype = dtype
-        self.acc_dtype = acc_dtype
-
-    def __setstate__(self, d):
-        super().__setstate__(d)
-        if not hasattr(self, "dtype"):
-            # This is needed as old pickled will crash otherwise.
-            # We need to keep the old dtype behavior as the op
-            # could be in an apply node with a specified dtype.
-            self.dtype = "OLD"
-
-        if not hasattr(self, "acc_dtype"):
-            # acc_dtype is not used by any external Op, so we do not
-            # need to keep the previous behaviour here.
-            self.acc_dtype = None
-
-    def _output_dtype(self, idtype):
-        dtype = self.dtype
-        if dtype == "OLD":
-            return dict(
-                int8="int32",
-                int16="int32",
-                int32="int64",
-                uint8="uint32",
-                uint16="uint32",
-                uint32="uint64",
-            ).get(idtype, idtype)
-        if dtype is None:
-            # If input has a discrete dtype, upcast it to 64
-            return dict(
-                bool="int64",
-                int8="int64",
-                int16="int64",
-                int32="int64",
-                uint8="uint64",
-                uint16="uint64",
-                uint32="uint64",
-            ).get(idtype, idtype)
-        else:
-            # The important is that the accumulator dtype does not
-            # lose precision. Then, the result can be downcasted.
-            return dtype
-
-    def _acc_dtype(self, idtype):
-        acc_dtype = self.acc_dtype
-        if acc_dtype is None:
-            return dict(
-                bool="int64",
-                int8="int64",
-                int16="int64",
-                int32="int64",
-                uint8="uint64",
-                uint16="uint64",
-                uint32="uint64",
-                float16="float32",
-                float32="float64",
-                complex64="complex128",
-            ).get(idtype, idtype)
-        elif acc_dtype in continuous_dtypes and idtype in discrete_dtypes:
-            # Specifying a continuous accumulator for discrete input is OK
-            return acc_dtype
-        else:
-            # The conversion has to be considered an upcast.
-            upcasted_dtype = upcast(idtype, acc_dtype)
-            if acc_dtype != upcasted_dtype:
-                raise TypeError(
-                    f"Cannot build {self} node with input dtype {idtype} "
-                    f"and acc_dtype {acc_dtype}, as precision would be lost. "
-                    "To correct this error, you can:\n"
-                    "  - not specify acc_dtype, or\n"
-                    f"  - use an acc_dtype at least as precise as {upcasted_dtype}.\n"
-                    '  - specify "dtype" instead of "acc_dtype", so '
-                    "the reduction will be precise, but the result will "
-                    'be casted into "dtype" at the end.\n'
-                    "If you are expecting the precision loss, you can "
-                    f'use tensor.cast(..., dtype="{acc_dtype}"), on your input.'
-                )
-            return acc_dtype
-
-    def make_node(self, input):
-        # We need to redefine make_node so that, if self.dtype is None,
-        # we can infer what dtype should be, and create a node from an Op
-        # of the appropriate dtype.
-        input = as_tensor_variable(input)
-        dtype = self._output_dtype(input.dtype)
-        acc_dtype = self._acc_dtype(input.dtype)
-
-        assert dtype is not None
-        assert acc_dtype is not None
-
-        if dtype == self.dtype and acc_dtype == self.acc_dtype:
-            # Don't build another instance
-            op = self
-        else:
-            op = copy(self)
-            op.set_ufunc(self.scalar_op)
-            op.dtype = dtype
-            op.acc_dtype = acc_dtype
-
-        assert op.acc_dtype is not None
-
-        # TODO: Why doesn't `make_node` just take these
-        # automatically-determined values as arguments?
-        return super(CAReduceDtype, op).make_node(input)
-
-    def __str__(self):
-        prefix = f"{type(self).__name__}{{{self.scalar_op}}}"
-        if self.axis is not None:
-            axis = ", ".join(str(x) for x in self.axis)
-            return f"{prefix}{{axis=[{axis}], acc_dtype={self.acc_dtype}}}"
-        else:
-            return f"{prefix}{{acc_dtype={self.acc_dtype}}}"
-
-
 def scalar_elemwise(*symbol, nfunc=None, nin=None, nout=None, symbolname=None):
     """Replace a symbol definition with an `Elemwise`-wrapped version of the corresponding scalar `Op`.
 
