@@ -23,6 +23,7 @@ from aesara.misc.safe_asarray import _asarray
 from aesara.raise_op import Assert
 from aesara.scalar import int32 as int_t
 from aesara.scalar import upcast
+from aesara.scalar.basic import Composite
 from aesara.tensor import basic as at
 from aesara.tensor import get_vector_length
 from aesara.tensor.exceptions import NotScalarConstantError
@@ -667,19 +668,21 @@ class Repeat(Op):
             )
 
         if self.axis is None:
-            broadcastable = [False]
+            out_shape = [None]
         else:
             try:
                 const_reps = at.get_scalar_constant_value(repeats)
             except NotScalarConstantError:
                 const_reps = None
             if const_reps == 1:
-                broadcastable = x.broadcastable
+                out_shape = x.type.shape
             else:
-                broadcastable = list(x.broadcastable)
-                broadcastable[self.axis] = False
+                out_shape = list(x.type.shape)
+                out_shape[self.axis] = None
 
-        out_type = TensorType(x.dtype, broadcastable)
+        out_type = TensorType(
+            x.dtype, shape=tuple(1 if s == 1 else None for s in out_shape)
+        )
 
         return Apply(self, [x, repeats], [out_type()])
 
@@ -1177,33 +1180,26 @@ class Unique(Op):
         self.return_inverse = return_inverse
         self.return_counts = return_counts
         self.axis = axis
-        numpy_ver = [int(n) for n in np.__version__.split(".")[:2]]
-        if self.axis is not None and bool(numpy_ver < [1, 13]):
-            raise RuntimeError(
-                "Numpy version = "
-                + np.__version__
-                + f". Option 'axis={axis}' works starting from version 1.13.0."
-            )
 
     def make_node(self, x):
         x = at.as_tensor_variable(x)
         self_axis = self.axis
         if self_axis is None:
-            broadcastable = [False]
+            out_shape = (None,)
         else:
             if self_axis < 0:
-                self_axis += len(x.broadcastable)
-            if self_axis < 0 or self_axis >= len(x.broadcastable):
-                raise RuntimeError(
-                    "Unique axis `{}` is outside of input ndim = "
-                    "{}.".format(self.axis, len(x.broadcastable))
+                self_axis += x.type.ndim
+            if self_axis < 0 or self_axis >= x.type.ndim:
+                raise ValueError(
+                    f"Unique axis {self.axis} is outside of input ndim = {x.type.ndim}"
                 )
-            broadcastable = [
-                b if axis != self_axis else False
-                for axis, b in enumerate(x.broadcastable)
-            ]
-        outputs = [TensorType(shape=broadcastable, dtype=x.dtype)()]
-        typ = TensorType(shape=[False], dtype="int64")
+            out_shape = tuple(
+                s if s == 1 and axis != self_axis else None
+                for axis, s in enumerate(x.type.shape)
+            )
+
+        outputs = [TensorType(dtype=x.dtype, shape=out_shape)()]
+        typ = TensorType(dtype="int64", shape=(None,))
         if self.return_index:
             outputs.append(typ())
         if self.return_inverse:
@@ -1309,7 +1305,7 @@ class UnravelIndex(Op):
             self,
             [indices, dims],
             [
-                TensorType(dtype="int64", shape=(False,) * indices.ndim)()
+                TensorType(dtype="int64", shape=(None,) * indices.type.ndim)()
                 for i in range(at.get_vector_length(dims))
             ],
         )
@@ -1388,7 +1384,7 @@ class RavelMultiIndex(Op):
         return Apply(
             self,
             multi_index + [dims],
-            [TensorType(dtype="int64", shape=(False,) * multi_index[0].ndim)()],
+            [TensorType(dtype="int64", shape=(None,) * multi_index[0].type.ndim)()],
         )
 
     def infer_shape(self, fgraph, node, input_shapes):
@@ -1493,7 +1489,7 @@ def broadcast_shape_iter(
             (one_at,) * (max_dims - len(a))
             + tuple(
                 one_at
-                if getattr(sh, "value", sh) == 1
+                if sh == 1 or isinstance(sh, Constant) and sh.value == 1
                 else (aes.as_scalar(sh) if not isinstance(sh, Variable) else sh)
                 for sh in a
             )
@@ -1552,16 +1548,32 @@ def broadcast_shape_iter(
                     # be broadcastable or equal to the one non-broadcastable
                     # constant `const_nt_shape_var`.
                     assert_dim = Assert("Could not broadcast dimensions")
+
+                    scalar_nonconst_nb_shapes = [
+                        at.scalar_from_tensor(s)
+                        if isinstance(s.type, TensorType)
+                        else s
+                        for s in nonconst_nb_shapes
+                    ]
+
+                    dummy_nonconst_nb_shapes = [
+                        aes.get_scalar_type(dtype=v.dtype)()
+                        for v in scalar_nonconst_nb_shapes
+                    ]
                     assert_cond = reduce(
                         aes.and_,
                         (
                             aes.or_(
                                 aes.eq(nbv, one_at), aes.eq(nbv, const_nt_shape_var)
                             )
-                            for nbv in nonconst_nb_shapes
+                            for nbv in dummy_nonconst_nb_shapes
                         ),
                     )
-                    bcast_dim = assert_dim(const_nt_shape_var, assert_cond)
+                    assert_cond_op = Composite(dummy_nonconst_nb_shapes, [assert_cond])
+
+                    bcast_dim = assert_dim(
+                        const_nt_shape_var, assert_cond_op(*scalar_nonconst_nb_shapes)
+                    )
                 else:
                     bcast_dim = const_nt_shape_var
             else:
@@ -1579,21 +1591,37 @@ def broadcast_shape_iter(
                     result_dims.append(maybe_non_bcast_shapes[0])
                     continue
 
+                scalar_maybe_non_bcast_shapes = [
+                    at.scalar_from_tensor(s) if isinstance(s.type, TensorType) else s
+                    for s in maybe_non_bcast_shapes
+                ]
+                dummy_maybe_non_bcast_shapes = [
+                    aes.get_scalar_type(dtype=v.dtype)()
+                    for v in scalar_maybe_non_bcast_shapes
+                ]
                 non_bcast_vec = [
                     aes.switch(aes.eq(nbv, 1), -one_at, nbv)
-                    for nbv in maybe_non_bcast_shapes
+                    for nbv in dummy_maybe_non_bcast_shapes
                 ]
                 dim_max = aes.abs(reduce(aes.scalar_maximum, non_bcast_vec))
+                dim_max_op = Composite(dummy_maybe_non_bcast_shapes, [dim_max])
+
+                dummy_dim_max = dim_max_op(*dummy_maybe_non_bcast_shapes)
 
                 assert_dim = Assert("Could not broadcast dimensions")
                 assert_cond = reduce(
                     aes.and_,
                     (
-                        aes.or_(aes.eq(nbv, -one_at), aes.eq(nbv, dim_max))
+                        aes.or_(aes.eq(nbv, -one_at), aes.eq(nbv, dummy_dim_max))
                         for nbv in non_bcast_vec
                     ),
                 )
-                bcast_dim = assert_dim(dim_max, assert_cond)
+                assert_cond_op = Composite(dummy_maybe_non_bcast_shapes, [assert_cond])
+
+                bcast_dim = assert_dim(
+                    dim_max_op(*scalar_maybe_non_bcast_shapes),
+                    assert_cond_op(*scalar_maybe_non_bcast_shapes),
+                )
 
             result_dims.append(bcast_dim)
 
@@ -1613,9 +1641,9 @@ class BroadcastTo(COp):
     def make_node(self, a, *shape):
         a = at.as_tensor_variable(a)
 
-        shape, bcast = at.infer_broadcastable(shape)
+        shape, static_shape = at.infer_static_shape(shape)
 
-        out = TensorType(dtype=a.type.dtype, shape=bcast)()
+        out = TensorType(dtype=a.type.dtype, shape=static_shape)()
 
         # Attempt to prevent in-place operations on this view-based output
         out.tag.indestructible = True
@@ -1637,11 +1665,14 @@ class BroadcastTo(COp):
         d_wrt_a = broadcast_to(dout, shape).sum(axis=new_dims)
 
         # Determine the dimensions that were broadcast
-        _, shape_bcast = at.infer_broadcastable(shape)
+        _, static_shape = at.infer_static_shape(shape)
+
+        # TODO: This needs to be performed at run-time when static shape
+        # information isn't available.
         bcast_sums = [
             i
-            for i, (a_b, s_b) in enumerate(zip(a.broadcastable, shape_bcast[-a.ndim :]))
-            if a_b and not s_b
+            for i, (a_s, s_s) in enumerate(zip(a.type.shape, static_shape[-a.ndim :]))
+            if a_s == 1 and s_s != 1
         ]
 
         if bcast_sums:
