@@ -1,6 +1,6 @@
 import builtins
 import warnings
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING, List, Optional, Tuple
 
 import numpy as np
 
@@ -25,13 +25,7 @@ from aesara.tensor.basic import (
     stack,
     switch,
 )
-from aesara.tensor.elemwise import (
-    CAReduce,
-    CAReduceDtype,
-    DimShuffle,
-    Elemwise,
-    scalar_elemwise,
-)
+from aesara.tensor.elemwise import CAReduce, DimShuffle, Elemwise, scalar_elemwise
 from aesara.tensor.shape import shape, specify_broadcastable
 from aesara.tensor.type import (
     DenseTensorType,
@@ -46,7 +40,7 @@ from aesara.tensor.type import (
 )
 from aesara.tensor.type_other import NoneConst
 from aesara.tensor.utils import as_list
-from aesara.tensor.var import TensorConstant, _tensor_py_operators
+from aesara.tensor.var import TensorConstant
 
 
 if TYPE_CHECKING:
@@ -151,13 +145,15 @@ class MaxAndArgmax(COp):
         # We keep the original broadcastable flags for dimensions on which
         # we do not perform the max / argmax.
         all_axes = set(self.axis)
-        broadcastable = [
-            b for i, b in enumerate(x.type.broadcastable) if i not in all_axes
-        ]
         inputs = [x]
+        out_shape = tuple(
+            1 if s == 1 else None
+            for i, s in enumerate(x.type.shape)
+            if i not in all_axes
+        )
         outputs = [
-            tensor(x.type.dtype, broadcastable, name="max"),
-            tensor("int64", broadcastable, name="argmax"),
+            tensor(x.type.dtype, shape=out_shape, name="max"),
+            tensor("int64", shape=out_shape, name="argmax"),
         ]
         return Apply(self, inputs, outputs)
 
@@ -375,10 +371,8 @@ class Argmax(COp):
 
         # We keep the original broadcastable flags for dimensions on which
         # we do not perform the argmax.
-        broadcastable = [
-            b for i, b in enumerate(x.type.broadcastable) if i not in all_axes
-        ]
-        outputs = [tensor("int64", broadcastable, name="argmax")]
+        out_shape = tuple(s for i, s in enumerate(x.type.shape) if i not in all_axes)
+        outputs = [tensor("int64", shape=out_shape, name="argmax")]
         return Apply(self, inputs, outputs)
 
     def prepare_node(self, node, storage_map, compute_map, impl):
@@ -633,6 +627,10 @@ class Max(NonZeroCAReduce):
     def __init__(self, axis):
         super().__init__(aes.scalar_maximum, axis)
 
+    def clone(self, **kwargs):
+        axis = kwargs.get("axis", self.axis)
+        return type(self)(axis=axis)
+
 
 class Min(NonZeroCAReduce):
     nfunc_spec = ("min", 1, 1)
@@ -640,6 +638,10 @@ class Min(NonZeroCAReduce):
     def __init__(self, axis):
         super().__init__(aes.scalar_minimum, axis)
 
+    def clone(self, **kwargs):
+        axis = kwargs.get("axis", self.axis)
+        return type(self)(axis=axis)
+
 
 def max(x, axis=None, keepdims=False):
     """
@@ -1384,6 +1386,16 @@ def gammal(k, x):
     """Lower incomplete gamma function."""
 
 
+@scalar_elemwise
+def hyp2f1(a, b, c, z):
+    """Gaussian hypergeometric function."""
+
+
+@scalar_elemwise
+def hyp2f1_der(a, b, c, z):
+    """Derivatives for Gaussian hypergeometric function."""
+
+
 @scalar_elemwise
 def j0(x):
     """Bessel function of the first kind of order 0."""
@@ -1416,7 +1428,7 @@ def iv(v, x):
 
 @scalar_elemwise
 def sigmoid(x):
-    """Logistic sigmoid function (1 / (1 + exp(x)), also known as expit or inverse logit"""
+    """Logistic sigmoid function (1 / (1 + exp(-x)), also known as expit or inverse logit"""
 
 
 expit = sigmoid
@@ -1445,17 +1457,11 @@ def real(z):
     """Return real component of complex-valued tensor `z`"""
 
 
-_tensor_py_operators.real = property(real)
-
-
 @scalar_elemwise
 def imag(z):
     """Return imaginary component of complex-valued tensor `z`"""
 
 
-_tensor_py_operators.imag = property(imag)
-
-
 @scalar_elemwise
 def angle(z):
     """Return polar-coordinate angle of complex-valued tensor `z`"""
@@ -1530,6 +1536,10 @@ class Mean(CAReduce):
   """
         )
 
+    def clone(self, **kwargs):
+        axis = kwargs.get("axis", self.axis)
+        return type(self)(axis=axis)
+
 
 # TODO: implement the grad. When done and tested, you can make this the default
 # version.
@@ -1598,7 +1608,7 @@ def mean(input, axis=None, dtype=None, op=False, keepdims=False, acc_dtype=None)
 
     # Cast shp into a float type
     # TODO Once we have a consistent casting policy, we could simply
-    # use true_div.
+    # use true_divide.
     if s.dtype in ("float16", "float32", "complex64"):
         shp = cast(shp, "float32")
     else:
@@ -1615,7 +1625,7 @@ def mean(input, axis=None, dtype=None, op=False, keepdims=False, acc_dtype=None)
 
     # This sequential division will possibly be optimized by Aesara:
     for i in axis:
-        s = true_div(s, shp[i])
+        s = true_divide(s, shp[i])
 
     # This can happen when axis is an empty list/tuple
     if s.dtype != shp.dtype and s.dtype in discrete_dtypes:
@@ -1687,7 +1697,7 @@ def var(input, axis=None, ddof=0, keepdims=False, corrected=False):
         shp = shape(input) - ddof
         v = sum((centered_input**two), axis=axis, keepdims=keepdims)
         for i in axis:
-            v = true_div(v, shp[i])
+            v = true_divide(v, shp[i])
 
     # use 'corrected_two_pass' algorithm
     if corrected:
@@ -1698,7 +1708,7 @@ def var(input, axis=None, ddof=0, keepdims=False, corrected=False):
             shp_inp = shape(input)
             error = sum(centered_input, axis=axis, keepdims=keepdims) ** 2
             for i in axis:
-                error = true_div(error, shp[i] * shp_inp[i])
+                error = true_divide(error, shp[i] * shp_inp[i])
         v = v - error
 
     v.name = "var"
@@ -1784,7 +1794,7 @@ def mul(a, *other_terms):
 
 
 @scalar_elemwise
-def true_div(a, b):
+def true_divide(a, b):
     """elementwise [true] division (inverse of multiplication)"""
     # see decorator for function body
 
@@ -1866,7 +1876,7 @@ pprint.assign(add, printing.OperatorPrinter("+", -2, "either"))
 pprint.assign(mul, printing.OperatorPrinter("*", -1, "either"))
 pprint.assign(sub, printing.OperatorPrinter("-", -2, "left"))
 pprint.assign(neg, printing.OperatorPrinter("-", 0, "either"))
-pprint.assign(true_div, printing.OperatorPrinter("/", -1, "left"))
+pprint.assign(true_divide, printing.OperatorPrinter("/", -1, "left"))
 pprint.assign(int_div, printing.OperatorPrinter("//", -1, "left"))
 pprint.assign(pow, printing.OperatorPrinter("**", 1, "right"))
 
@@ -1911,15 +1921,14 @@ class Dot(Op):
                 "aesara.tensor.dot instead."
             )
 
-        i_broadcastables = [input.type.broadcastable for input in inputs]
-        bx, by = i_broadcastables
-        if len(by) == 2:  # y is a matrix
-            bz = bx[:-1] + by[-1:]
-        elif len(by) == 1:  # y is vector
-            bz = bx[:-1]
+        sx, sy = [input.type.shape for input in inputs]
+        if len(sy) == 2:
+            sz = sx[:-1] + sy[-1:]
+        elif len(sy) == 1:
+            sz = sx[:-1]
 
         i_dtypes = [input.type.dtype for input in inputs]
-        outputs = [tensor(aes.upcast(*i_dtypes), bz)]
+        outputs = [tensor(aes.upcast(*i_dtypes), shape=sz)]
         return Apply(self, inputs, outputs)
 
     def perform(self, node, inp, out):
@@ -2351,7 +2360,6 @@ class All(CAReduce):
 
     """
 
-    __props__ = ("axis",)
     nfunc_spec = ("all", 1, 1)
 
     def __init__(self, axis=None):
@@ -2377,6 +2385,10 @@ class All(CAReduce):
         (x,) = inp
         return [x.zeros_like(config.floatX)]
 
+    def clone(self, **kwargs):
+        axis = kwargs.get("axis", self.axis)
+        return type(self)(axis=axis)
+
 
 class Any(CAReduce):
     """Applies `bitwise or` to all the values of a tensor along the
@@ -2384,7 +2396,6 @@ class Any(CAReduce):
 
     """
 
-    __props__ = ("axis",)
     nfunc_spec = ("any", 1, 1)
 
     def __init__(self, axis=None):
@@ -2410,48 +2421,31 @@ class Any(CAReduce):
         (x,) = inp
         return [x.zeros_like(config.floatX)]
 
+    def clone(self, **kwargs):
+        axis = kwargs.get("axis", self.axis)
+        return type(self)(axis=axis)
 
-class Sum(CAReduceDtype):
+
+class Sum(CAReduce):
     """
     Sums all the values of a tensor along the specified axis(es).
 
-    Equivalent to `CAReduceDtype(scalar.add, axis=axis, dtype=dtype)`,
+    Equivalent to `CAReduce(scalar.add, axis=axis, dtype=dtype)`,
     with the difference that this defines the gradient of sum wrt its
     tensor input.
 
-    Parameters
-    ----------
-    axis
-        Axis(es) along which the tensor should be summed
-        (use None to sum over all axes, and a list or tuple to sum along more
-        than one axis).
-
-    dtype
-        The dtype of the internal accumulator and returned
-        tensor. If None, then we use the default dtype which is the same as the
-        input tensor's dtype except when:
-        - the input dtype is a signed integer of precision < 64 bit, in
-        which case we use int64
-        - the input dtype is an unsigned integer of precision < 64 bit, in
-        which case we use uint64
-        This value does not depend on the value of "acc_dtype".
-
-    acc_dtype
-        The dtype of the internal accumulator.
-        If None (default), we use the dtype in the list below,
-        or the input dtype if its precision is higher:
-        - for int dtypes, we use at least int64;
-        - for uint dtypes, we use at least uint64;
-        - for float dtypes, we use at least float64;
-        - for complex dtypes, we use at least complex128.
-
     """
 
-    __props__ = ("axis", "dtype", "acc_dtype")
     nfunc_spec = ("sum", 1, 1)
 
     def __init__(self, axis=None, dtype=None, acc_dtype=None):
-        super().__init__(aes.add, axis=axis, dtype=dtype, acc_dtype=acc_dtype)
+        super().__init__(
+            aes.add,
+            axis=axis,
+            dtype=dtype,
+            acc_dtype=acc_dtype,
+            upcast_discrete_output=True,
+        )
 
     def __str__(self):
         name = self.__class__.__name__
@@ -2493,6 +2487,12 @@ class Sum(CAReduceDtype):
             return [None]
         return self(*eval_points, return_list=True)
 
+    def clone(self, **kwargs):
+        axis = kwargs.get("axis", self.axis)
+        dtype = kwargs.get("dtype", self.dtype)
+        acc_dtype = kwargs.get("acc_dtype", self.acc_dtype)
+        return type(self)(axis=axis, dtype=dtype, acc_dtype=acc_dtype)
+
 
 def sum(input, axis=None, dtype=None, keepdims=False, acc_dtype=None):
     """
@@ -2524,7 +2524,7 @@ def sum(input, axis=None, dtype=None, keepdims=False, acc_dtype=None):
 pprint.assign(Sum, printing.FunctionPrinter(["sum"], ["axis"]))
 
 
-class Prod(CAReduceDtype):
+class Prod(CAReduce):
     """
     Multiplies all the values of a tensor along the specified axis(es).
 
@@ -2534,19 +2534,20 @@ class Prod(CAReduceDtype):
 
     """
 
-    __props__ = ("axis", "dtype", "acc_dtype")
+    __props__ = ("scalar_op", "axis", "dtype", "acc_dtype", "no_zeros_in_input")
+
     nfunc_spec = ("prod", 1, 1)
 
     def __init__(self, axis=None, dtype=None, acc_dtype=None, no_zeros_in_input=False):
-        super().__init__(aes.mul, axis=axis, dtype=dtype, acc_dtype=acc_dtype)
+        super().__init__(
+            aes.mul,
+            axis=axis,
+            dtype=dtype,
+            acc_dtype=acc_dtype,
+            upcast_discrete_output=True,
+        )
         self.no_zeros_in_input = no_zeros_in_input
 
-    def __setstate__(self, dct):
-        super().__setstate__(dct)
-        # Add default value to be able to reload old pickled objects.
-        if "no_zeros_in_input" not in dct:
-            self.no_zeros_in_input = False
-
     def L_op(self, inp, out, grads):
         """
         The grad of this Op could be very easy, if it is was not for the case
@@ -2669,6 +2670,18 @@ class Prod(CAReduceDtype):
     def c_code_cache_version(self):
         return (1,)
 
+    def clone(self, **kwargs):
+        axis = kwargs.get("axis", self.axis)
+        dtype = kwargs.get("dtype", self.dtype)
+        acc_dtype = kwargs.get("acc_dtype", self.acc_dtype)
+        no_zeros_in_input = kwargs.get("no_zeros_in_input", self.no_zeros_in_input)
+        return type(self)(
+            axis=axis,
+            dtype=dtype,
+            acc_dtype=acc_dtype,
+            no_zeros_in_input=no_zeros_in_input,
+        )
+
 
 def prod(
     input,
@@ -2737,12 +2750,15 @@ class MulWithoutZeros(BinaryScalarOp):
 mul_without_zeros = MulWithoutZeros(aes.upcast_out, name="mul_without_zeros")
 
 
-class ProdWithoutZeros(CAReduceDtype):
-
-    __props__ = ("axis", "dtype", "acc_dtype")
-
+class ProdWithoutZeros(CAReduce):
     def __init__(self, axis=None, dtype=None, acc_dtype=None):
-        super().__init__(mul_without_zeros, axis=axis, dtype=dtype, acc_dtype=acc_dtype)
+        super().__init__(
+            mul_without_zeros,
+            axis=axis,
+            dtype=dtype,
+            acc_dtype=acc_dtype,
+            upcast_discrete_output=True,
+        )
 
     def grad(self, inp, grads):
         from aesara.gradient import grad_not_implemented
@@ -2758,6 +2774,12 @@ class ProdWithoutZeros(CAReduceDtype):
         )
         return [a_grad]
 
+    def clone(self, **kwargs):
+        axis = kwargs.get("axis", self.axis)
+        dtype = kwargs.get("dtype", self.dtype)
+        acc_dtype = kwargs.get("acc_dtype", self.acc_dtype)
+        return type(self)(axis=axis, dtype=dtype, acc_dtype=acc_dtype)
+
 
 def any(x, axis=None, keepdims=False):
     out = Any(axis)(x)
@@ -3099,7 +3121,7 @@ __all__ = [
     "add",
     "sub",
     "mul",
-    "true_div",
+    "true_divide",
     "int_div",
     "floor_div",
     "ceil_intdiv",
@@ -3116,11 +3138,18 @@ __all__ = [
     "power",
     "logaddexp",
     "logsumexp",
+    "hyp2f1",
+    "hyp2f1_der",
 ]
 
-DEPRECATED_NAMES = [
+DEPRECATED_NAMES: List[Tuple[str, str, object]] = [
     ("abs_", "`abs_` is deprecated; use `abs` instead.", abs),
     ("inv", "`inv` is deprecated; use `reciprocal` instead.", reciprocal),
+    (
+        "true_div",
+        "`true_div` is deprecated; use `true_divide` or `divide` instead.",
+        true_divide,
+    ),
 ]
 
 
