@@ -94,6 +94,11 @@ class UniformRV(RandomVariable):
 
         The results are undefined when `high < low`.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         low
@@ -141,6 +146,11 @@ class TriangularRV(RandomVariable):
     def __call__(self, left, mode, right, size=None, **kwargs):
         r"""Draw samples from a triangular distribution.
 
+        Signature
+        ---------
+
+        `(), (), () -> ()`
+
         Parameters
         ----------
         left
@@ -191,6 +201,11 @@ class BetaRV(RandomVariable):
     def __call__(self, alpha, beta, size=None, **kwargs):
         r"""Draw samples from a beta distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         alpha
@@ -210,6 +225,48 @@ class BetaRV(RandomVariable):
 beta = BetaRV()
 
 
+class RayleighRV(RandomVariable):
+    r"""A rayleigh continuous random variable.
+
+    The probability density function for `rayleigh` is:
+
+    .. math::
+
+        P(x;scale) = \\frac{x}{scale^2}e^{\\frac{-x^2}{2 \\cdotp scale^2}}
+
+
+    """
+    name = "rayleigh"
+    ndim_supp = 0
+    ndims_params = [0]
+    dtype = "floatX"
+    _print_name = ("Rayleigh", "\\operatorname{Rayleigh}")
+
+    def __call__(self, scale=1.0, size=None, **kwargs):
+        r"""Draw samples from a rayleigh distribution.
+
+        Signature
+        ---------
+
+        `() -> ()`
+
+        Parameters
+        ----------
+        scale
+            Scale, also equals the mode. Must be non-negative. Default is 1
+        size
+            Sample shape. If the given size is, e.g. `(m, n, k)` then `m * n * k`
+            independent, identically distributed random variables are
+            returned. Default is `None` in which case a single random variable
+            is returned.
+
+        """
+        return super().__call__(scale, size=size, **kwargs)
+
+
+rayleigh = RayleighRV()
+
+
 class NormalRV(RandomVariable):
     r"""A normal continuous random variable.
 
@@ -232,6 +289,11 @@ class NormalRV(RandomVariable):
     def __call__(self, loc=0.0, scale=1.0, size=None, **kwargs):
         r"""Draw samples from a normal distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         loc
@@ -265,6 +327,11 @@ class StandardNormalRV(NormalRV):
     def __call__(self, size=None, **kwargs):
         """Draw samples from a standard normal distribution.
 
+        Signature
+        ---------
+
+        `nil -> ()`
+
         Parameters
         ----------
         size
@@ -302,6 +369,11 @@ class HalfNormalRV(ScipyRandomVariable):
     def __call__(self, loc=0.0, scale=1.0, size=None, **kwargs):
         r"""Draw samples from a half-normal distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         loc
@@ -362,6 +434,11 @@ class LogNormalRV(RandomVariable):
     def __call__(self, mean=0.0, sigma=1.0, size=None, **kwargs):
         r"""Draw sample from a lognormal distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         mean
@@ -408,6 +485,11 @@ class GammaRV(ScipyRandomVariable):
     def __call__(self, shape, rate, size=None, **kwargs):
         r"""Draw samples from a gamma distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         shape
@@ -461,6 +543,11 @@ class ChiSquareRV(RandomVariable):
     def __call__(self, df, size=None, **kwargs):
         r"""Draw samples from a chisquare distribution.
 
+        Signature
+        ---------
+
+        `() -> ()`
+
         Parameters
         ----------
         df
@@ -500,6 +587,11 @@ class ParetoRV(ScipyRandomVariable):
     def __call__(self, b, scale=1.0, size=None, **kwargs):
         r"""Draw samples from a pareto distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         b
@@ -551,6 +643,11 @@ class GumbelRV(ScipyRandomVariable):
     ) -> RandomVariable:
         r"""Draw samples from a gumbel distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         loc
@@ -601,6 +698,11 @@ class ExponentialRV(RandomVariable):
     def __call__(self, scale=1.0, size=None, **kwargs):
         r"""Draw samples from an exponential distribution.
 
+        Signature
+        ---------
+
+        `() -> ()`
+
         Parameters
         ----------
         scale
@@ -639,6 +741,11 @@ class WeibullRV(RandomVariable):
     def __call__(self, shape, size=None, **kwargs):
         r"""Draw samples from a weibull distribution.
 
+        Signature
+        ---------
+
+        `() -> ()`
+
         Parameters
         ----------
         shape
@@ -678,6 +785,12 @@ class LogisticRV(RandomVariable):
     def __call__(self, loc=0, scale=1, size=None, **kwargs):
         r"""Draw samples from a logistic distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
+
         Parameters
         ----------
         loc
@@ -720,6 +833,11 @@ class VonMisesRV(RandomVariable):
     def __call__(self, mu, kappa, size=None, **kwargs):
         r"""Draw samples from a von Mises distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         mu
@@ -782,6 +900,11 @@ class MvNormalRV(RandomVariable):
     def __call__(self, mean=None, cov=None, size=None, **kwargs):
         r""" "Draw samples from a multivariate normal distribution.
 
+        Signature
+        ---------
+
+        `(n), (n,n) -> (n)`
+
         Parameters
         ----------
         mean
@@ -856,6 +979,11 @@ class DirichletRV(RandomVariable):
     def __call__(self, alphas, size=None, **kwargs):
         r"""Draw samples from a dirichlet distribution.
 
+        Signature
+        ---------
+
+        `(k) -> (k)`
+
         Parameters
         ----------
         alphas
@@ -916,6 +1044,11 @@ class PoissonRV(RandomVariable):
     def __call__(self, lam=1.0, size=None, **kwargs):
         r"""Draw samples from a poisson distribution.
 
+        Signature
+        ---------
+
+        `() -> ()`
+
         Parameters
         ----------
         lam
@@ -956,6 +1089,11 @@ class GeometricRV(RandomVariable):
     def __call__(self, p, size=None, **kwargs):
         r"""Draw samples from a geometric distribution.
 
+        Signature
+        ---------
+
+        `() -> ()`
+
         Parameters
         ----------
         p
@@ -993,6 +1131,11 @@ class HyperGeometricRV(RandomVariable):
     def __call__(self, ngood, nbad, nsample, size=None, **kwargs):
         r"""Draw samples from a geometric distribution.
 
+        Signature
+        ---------
+
+        `(), (), () -> ()`
+
         Parameters
         ----------
         ngood
@@ -1036,6 +1179,11 @@ class CauchyRV(ScipyRandomVariable):
     def __call__(self, loc=0.0, scale=1.0, size=None, **kwargs):
         r"""Draw samples from a Cauchy distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         loc
@@ -1081,6 +1229,11 @@ class HalfCauchyRV(ScipyRandomVariable):
     def __call__(self, loc=0.0, scale=1.0, size=None, **kwargs):
         r"""Draw samples from a half-Cauchy distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         loc
@@ -1130,6 +1283,11 @@ class InvGammaRV(ScipyRandomVariable):
     def __call__(self, shape, scale, size=None, **kwargs):
         r"""Draw samples from an inverse-gamma distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         shape
@@ -1175,6 +1333,11 @@ class WaldRV(RandomVariable):
     def __call__(self, mean=1.0, scale=1.0, size=None, **kwargs):
         r"""Draw samples from a Wald distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         mean
@@ -1217,6 +1380,11 @@ class TruncExponentialRV(ScipyRandomVariable):
     def __call__(self, b, loc=0.0, scale=1.0, size=None, **kwargs):
         r"""Draw samples from a truncated exponential distribution.
 
+        Signature
+        ---------
+
+        `(), (), () -> ()`
+
         Parameters
         ----------
         b
@@ -1244,6 +1412,60 @@ class TruncExponentialRV(ScipyRandomVariable):
 truncexpon = TruncExponentialRV()
 
 
+class StudentTRV(ScipyRandomVariable):
+    r"""A Student's t continuous random variable.
+
+    The probability density function for `t` in terms of its degrees of freedom
+    parameter :math:`\nu`, location parameter :math:`\mu` and scale
+    parameter :math:`\sigma` is:
+
+    .. math::
+
+        f(x; \nu, \alpha, \beta) = \frac{\Gamma(\frac{\nu + 1}{2})}{\Gamma(\frac{\nu}{2})} \left(\frac{1}{\pi\nu\sigma}\right)^{\frac{1}{2}} \left[1+\frac{(x-\mu)^2}{\nu\sigma}\right]^{-\frac{\nu+1}{2}}
+
+    for :math:`\nu > 0`, :math:`\sigma > 0`.
+
+    """
+    name = "t"
+    ndim_supp = 0
+    ndims_params = [0, 0, 0]
+    dtype = "floatX"
+    _print_name = ("StudentT", "\\operatorname{StudentT}")
+
+    def __call__(self, df, loc=0.0, scale=1.0, size=None, **kwargs):
+        r"""Draw samples from a Student's t distribution.
+
+        Signature
+        ---------
+
+        `(), (), () -> ()`
+
+        Parameters
+        ----------
+        df
+            Degrees of freedom parameter :math:`\nu` of the distribution. Must be
+            positive.
+        loc
+            Location parameter :math:`\mu` of the distribution.
+        scale
+            Scale parameter :math:`\sigma` of the distribution. Must be
+            positive.
+        size
+           Sample shape. If the given size is `(m, n, k)`, then `m * n * k`
+           independent, identically distributed samples are returned. Default is
+           `None` in which case a single sample is returned.
+
+        """
+        return super().__call__(df, loc, scale, size=size, **kwargs)
+
+    @classmethod
+    def rng_fn_scipy(cls, rng, df, loc, scale, size):
+        return stats.t.rvs(df, loc=loc, scale=scale, size=size, random_state=rng)
+
+
+t = StudentTRV()
+
+
 class BernoulliRV(ScipyRandomVariable):
     r"""A Bernoulli discrete random variable.
 
@@ -1272,6 +1494,11 @@ class BernoulliRV(ScipyRandomVariable):
     def __call__(self, p, size=None, **kwargs):
         r"""Draw samples from a Bernoulli distribution.
 
+        Signature
+        ---------
+
+        `() -> ()`
+
         Parameters
         ----------
         p
@@ -1314,6 +1541,12 @@ class LaplaceRV(RandomVariable):
     def __call__(self, loc=0.0, scale=1.0, size=None, **kwargs):
         r"""Draw samples from a Laplace distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
+
         Parameters
         ----------
         loc
@@ -1354,6 +1587,11 @@ class BinomialRV(RandomVariable):
     def __call__(self, n, p, size=None, **kwargs):
         r"""Draw samples from a binomial distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         n
@@ -1393,6 +1631,11 @@ class NegBinomialRV(ScipyRandomVariable):
     def __call__(self, n, p, size=None, **kwargs):
         r"""Draw samples from a negative binomial distribution.
 
+        Signature
+        ---------
+
+        `(), () -> ()`
+
         Parameters
         ----------
         n
@@ -1443,6 +1686,11 @@ class BetaBinomialRV(ScipyRandomVariable):
     def __call__(self, n, a, b, size=None, **kwargs):
         r"""Draw samples from a beta-binomial distribution.
 
+        Signature
+        ---------
+
+        `(), (), () -> ()`
+
         Parameters
         ----------
         n
@@ -1489,6 +1737,11 @@ class GenGammaRV(ScipyRandomVariable):
     def __call__(self, alpha=1.0, p=1.0, lambd=1.0, size=None, **kwargs):
         r"""Draw samples from a generalized gamma distribution.
 
+        Signature
+        ---------
+
+        `(), (), () -> ()`
+
         Parameters
         ----------
         alpha
@@ -1546,6 +1799,11 @@ class MultinomialRV(RandomVariable):
     def __call__(self, n, p, size=None, **kwargs):
         r"""Draw samples from a discrete multinomial distribution.
 
+        Signature
+        ---------
+
+        `(), (n) -> (n)`
+
         Parameters
         ----------
         n
@@ -1613,6 +1871,11 @@ class CategoricalRV(RandomVariable):
     def __call__(self, p, size=None, **kwargs):
         r"""Draw samples from a discrete categorical distribution.
 
+        Signature
+        ---------
+
+        `(j) -> ()`
+
         Parameters
         ----------
         p
@@ -1664,6 +1927,11 @@ class RandIntRV(RandomVariable):
     def __call__(self, low, high=None, size=None, **kwargs):
         r"""Draw samples from a discrete uniform distribution.
 
+        Signature
+        ---------
+
+        `() -> ()`
+
         Parameters
         ----------
         low
@@ -1711,6 +1979,11 @@ class IntegersRV(RandomVariable):
     def __call__(self, low, high=None, size=None, **kwargs):
         r"""Draw samples from a discrete uniform distribution.
 
+        Signature
+        ---------
+
+        `() -> ()`
+
         Parameters
         ----------
         low
@@ -1771,6 +2044,11 @@ class ChoiceRV(RandomVariable):
     def __call__(self, a, size=None, replace=True, p=None, **kwargs):
         r"""Generate a random sample from an array.
 
+        Signature
+        ---------
+
+        `(x) -> ()`
+
         Parameters
         ----------
         a
@@ -1781,7 +2059,7 @@ class ChoiceRV(RandomVariable):
             k` independent samples are returned. Default is `None`, in
             which case a single sample is returned.
         replace
-            When ``True``, sampling is performed with replacement.
+            When `True`, sampling is performed with replacement.
         p
             The probabilities associated with each entry in `a`. If not
             given, all elements have equal probability.
@@ -1831,6 +2109,11 @@ class PermutationRV(RandomVariable):
     def __call__(self, x, **kwargs):
         r"""Randomly permute a sequence or a range of values.
 
+        Signature
+        ---------
+
+        `(x) -> (x)`
+
         Parameters
         ----------
         x
@@ -1845,6 +2128,9 @@ class PermutationRV(RandomVariable):
 permutation = PermutationRV()
 
 
+random = uniform
+
+
 __all__ = [
     "permutation",
     "choice",
@@ -1884,4 +2170,7 @@ __all__ = [
     "standard_normal",
     "negative_binomial",
     "gengamma",
+    "t",
+    "random",
+    "rayleigh",
 ]
