@@ -364,8 +364,8 @@ class ShapeFeature(Feature):
                 else:
                     shape_vars.append(self.unpack(s[i], r))
             assert all(
-                not hasattr(r.type, "broadcastable")
-                or not r.type.broadcastable[i]
+                not hasattr(r.type, "shape")
+                or r.type.shape[i] != 1
                 or self.lscalar_one.equals(shape_vars[i])
                 or self.lscalar_one.equals(extract_constant(shape_vars[i]))
                 for i in range(r.type.ndim)
@@ -447,9 +447,9 @@ class ShapeFeature(Feature):
                 merged_shape.append(other_shape[i])
         assert all(
             (
-                not hasattr(r.type, "broadcastable")
-                or not r.type.broadcastable[i]
-                and not other_r.type.broadcastable[i]
+                not hasattr(r.type, "shape")
+                or r.type.shape[i] != 1
+                and other_r.type.shape[i] != 1
             )
             or self.lscalar_one.equals(merged_shape[i])
             or self.lscalar_one.equals(
@@ -474,8 +474,8 @@ class ShapeFeature(Feature):
             else:
                 new_shape.append(s_j)
         assert all(
-            not hasattr(r.type, "broadcastable")
-            or not r.type.broadcastable[idx]
+            not hasattr(r.type, "shape")
+            or r.type.shape[idx] != 1
             or self.lscalar_one.equals(new_shape[idx])
             or self.lscalar_one.equals(extract_constant(new_shape[idx]))
             for idx in range(r.type.ndim)
@@ -781,7 +781,11 @@ def local_reshape_chain(op):
         # We should try to figure out why we lost the information about this
         # constant value... but in the meantime, better not apply this
         # rewrite.
-        if rval.broadcastable == node.outputs[0].broadcastable:
+        if rval.type.ndim == node.outputs[0].type.ndim and all(
+            s1 == s1
+            for s1, s2 in zip(rval.type.shape, node.outputs[0].type.shape)
+            if s1 == 1 or s2 == 1
+        ):
             return [rval]
         else:
             return False
@@ -797,27 +801,31 @@ register_canonicalize(local_reshape_chain(Reshape), name="local_reshape_chain")
 @register_stabilize
 @node_rewriter([Reshape])
 def local_useless_reshape(fgraph, node):
-    """
-    Remove two kinds of useless reshape.
+    """Remove two kinds of useless `Reshape`.
 
-    Remove Reshape when both the input and output have a single dimension.
-    Remove Reshape when reshaping to the shape of the input.
+    - Remove `Reshape` when both the input and output have a single dimension.
+    - Remove `Reshape` when reshaping to the shape of the input.
 
     """
-    op = node.op
-    if not isinstance(op, Reshape):
-        return False
-
     inp = node.inputs[0]
     output = node.outputs[0]
     output_shape = node.inputs[1]
 
-    if inp.ndim != output.ndim:
+    if inp.type.ndim != output.type.ndim:
         return False
 
     # Simple case: both input and output have a single dimension.
-    # This could hide errors if the user provides inconsistent shapes.
-    if inp.ndim == 1 and output.ndim == 1 and inp.broadcastable == output.broadcastable:
+    # TODO FIXME XXX: This could hide errors if the user provides inconsistent
+    # shapes.
+    if (
+        inp.type.ndim == 1
+        and output.type.ndim == 1
+        and all(
+            s1 == s2
+            for s1, s2 in zip(inp.type.shape, output.type.shape)
+            if s1 == 1 or s2 == 1
+        )
+    ):
         return [inp]
 
     # Second case: all the shapes match the input shape
@@ -835,8 +843,8 @@ def local_useless_reshape(fgraph, node):
         shape_feature = getattr(fgraph, "shape_feature", None)
 
         nb_m1 = 0
-        shape_match = [False] * inp.ndim
-        for dim in range(inp.ndim):
+        shape_match = [False] * inp.type.ndim
+        for dim in range(inp.type.ndim):
             outshp_i = output_shape_is[dim]
             # Match Shape_i{dim}(input)
             if (
@@ -862,9 +870,9 @@ def local_useless_reshape(fgraph, node):
                         shape_match[dim] = True
                         continue
 
-            # Match 1 if input.broadcastable[dim] is True
+            # Match 1 if input.type.shape[dim] == 1
             cst_outshp_i = extract_constant(outshp_i, only_process_constants=1)
-            if inp.broadcastable[dim] and cst_outshp_i == 1:
+            if inp.type.shape[dim] == 1 and cst_outshp_i == 1:
                 shape_match[dim] = True
                 continue
 
@@ -895,22 +903,18 @@ def local_useless_reshape(fgraph, node):
 @register_canonicalize
 @node_rewriter([Reshape])
 def local_reshape_to_dimshuffle(fgraph, node):
-    """
-    Broadcastable dimensions in Reshape are replaced with dimshuffle.
+    r"""Replace broadcastable dimensions in `Reshape` nodes with `DimShuffle`\s.
 
-    The goal is to avoid using reshape to add or remove broadcastable
-    dimensions, but use dimshuffle instead, so dimshuffles can cancel out
-    or be removed later on.
+    The goal is to avoid using `Reshape` to add or remove broadcastable
+    dimensions, and to use `DimShuffle` instead, since `DimShuffle`\s can
+    cancel out and/or be removed later on.
 
     For example:
-        - reshape(x, (1, n)) --> dimshuffle{x,0}(reshape(x, (n,))
+        - reshape(x, (1, n)) -> DimShuffle{x,0}(Reshape(x, (n,))
         - reshape(x, (1, m, 1, n, 1, 1))
-          --> dimshuffle{x,0,x,1,x,x}(reshape(x, (m, n)))
+          -> DimShuffle{x,0,x,1,x,x}(Reshape(x, (m, n)))
     """
     op = node.op
-    if not isinstance(op, Reshape):
-        return False
-
     inp = node.inputs[0]
     output = node.outputs[0]
     output_shape = node.inputs[1]
@@ -931,10 +935,15 @@ def local_reshape_to_dimshuffle(fgraph, node):
             dimshuffle_new_order.append(index)
             new_output_shape.append(dim)
             index = index + 1
-    if index != output.ndim:
+
+    if index != output.type.ndim:
         inner = op.__class__(len(new_output_shape))(inp, new_output_shape)
         copy_stack_trace(output, inner)
-        new_node = [DimShuffle(inner.type.broadcastable, dimshuffle_new_order)(inner)]
+        new_node = [
+            DimShuffle(tuple(s == 1 for s in inner.type.shape), dimshuffle_new_order)(
+                inner
+            )
+        ]
         copy_stack_trace(output, new_node)
         return new_node
 
@@ -1023,8 +1032,8 @@ def local_Shape_of_SpecifyShape(fgraph, node):
 @register_useless
 @register_canonicalize
 @node_rewriter([Shape_i])
-def local_Shape_i_of_broadcastable(fgraph, node):
-    """Replace ``shape_i(x, i)`` with ``1`` when ``x.broadcastable[i]`` is ``True``."""
+def local_Shape_i_ground(fgraph, node):
+    """Replace ``shape_i(x, i)`` with ``s`` when ``x.type.shape[i] == s``."""
 
     if not isinstance(node.op, Shape_i):
         return False
@@ -1034,8 +1043,9 @@ def local_Shape_i_of_broadcastable(fgraph, node):
     if not isinstance(shape_arg.type, TensorType):
         return False
 
-    if shape_arg.broadcastable[node.op.i]:
-        return [as_tensor_variable(1, dtype=np.int64)]
+    s_val = shape_arg.type.shape[node.op.i]
+    if s_val is not None:
+        return [as_tensor_variable(s_val, dtype=np.int64)]
 
 
 @register_specialize
@@ -1098,10 +1108,9 @@ def local_useless_dimshuffle_in_reshape(fgraph, node):
 
     new_order = node.inputs[0].owner.op.new_order
     inp = node.inputs[0].owner.inputs[0]
-    broadcastables = node.inputs[0].broadcastable
     new_order_of_nonbroadcast = []
-    for i, bd in zip(new_order, broadcastables):
-        if not bd:
+    for i, s in zip(new_order, node.inputs[0].type.shape):
+        if s != 1:
             new_order_of_nonbroadcast.append(i)
     no_change_in_order = all(
         new_order_of_nonbroadcast[i] <= new_order_of_nonbroadcast[i + 1]
@@ -1125,7 +1134,11 @@ def local_useless_unbroadcast(fgraph, node):
     """
     if isinstance(node.op, Unbroadcast):
         x = node.inputs[0]
-        if x.broadcastable == node.outputs[0].broadcastable:
+        if x.type.ndim == node.outputs[0].type.ndim and all(
+            s1 == s2
+            for s1, s2 in zip(x.type.shape, node.outputs[0].type.shape)
+            if s1 == 1 or s2 == 1
+        ):
             # No broadcastable flag was modified
             # No need to copy over stack trace,
             # because x should already have a stack trace.
