@@ -19,6 +19,14 @@ from aesara.scalar import bool as bool_t
 from aesara.tensor.type import TensorType, int_dtypes
 
 
+warnings.warn(
+    "The module `aesara.tensor.signal` is deprecated and will "
+    "be removed from Aesara in version 2.8.5.",
+    DeprecationWarning,
+    stacklevel=2,
+)
+
+
 def max_pool_2d_same_size(input, patch_size):
     """
     Takes as input a 4-D tensor. It sets all non maximum values
@@ -534,8 +542,10 @@ class Pool(OpenMPOp):
         if pad.dtype not in int_dtypes:
             raise TypeError("Padding parameters must be ints.")
         # If the input shape are broadcastable we can have 0 in the output shape
-        broad = x.broadcastable[:-nd] + (False,) * nd
-        out = TensorType(x.dtype, broad)
+        out_shape = tuple(
+            1 if s == 1 else None for s in x.type.shape[:-nd] + (None,) * nd
+        )
+        out = TensorType(x.dtype, shape=out_shape)
         return Apply(self, [x, ws, stride, pad], [out()])
 
     def perform(self, node, inp, out, params):
@@ -2200,8 +2210,10 @@ class MaxPoolRop(OpenMPOp):
         if not pad.dtype.startswith("int"):
             raise TypeError("Padding parameters must be ints.")
         # If the input shape are broadcastable we can have 0 in the output shape
-        broad = x.broadcastable[:-nd] + (False,) * nd
-        out = TensorType(eval_point.dtype, broad)
+        out_shape = tuple(
+            1 if s == 1 else None for s in x.type.shape[:-nd] + (None,) * nd
+        )
+        out = TensorType(eval_point.dtype, shape=out_shape)
         return Apply(self, [x, eval_point, ws, stride, pad], [out()])
 
     def perform(self, node, inp, out, params):
